#include "tablet.h"
#include <stdio.h>
#include <string.h>

BTSWacomTablet::BTSWacomTablet(const char *port)
{
	fPort.SetDataRate(B_9600_BPS);
	fPort.SetDataBits(B_DATA_BITS_8);
	fPort.SetStopBits(B_STOP_BITS_1);
	fPort.SetParityMode(B_NO_PARITY);
	fPort.SetFlowControl(0); // but soft and B_HARDWARE_CONTROL also work
	fPort.SetBlocking(true); 
	fPortStatus = fPort.Open(port);
	fPort.SetTimeout(250000);		// Timeout of quarter a second
	snooze(100000);
}

BTSWacomTablet::~BTSWacomTablet()
{

	fPort.Close();
}

void	
BTSWacomTablet::SendCommand(const char *cmd)
{
	fPort.Write(cmd,strlen(cmd));
}

//
// Retrieve a line of data from the tablet.  It should be
// terminated with a <cr>
void
BTSWacomTablet::GetLine(char *buff, const long buffLen)
{
	//	return;
	
	bool done = false;
	char *ptr = buff;
	uint8 aChar;
	long bytestoread=0;
	long totalread = 0;
	
//	while(!done && (totalread < buffLen))
//	{
		bytestoread = fPort.WaitForInput();
		//printf("BytesToRead - %d\n", bytestoread);
		while(bytestoread)
		{
			long numRead = fPort.Read(&aChar, 1);
			if (numRead>0)
			{
			//printf("%c",aChar);
			//fflush(stdout);
				if (aChar == '\r')
				{
					*ptr = '\0';
					done = true;
					break;
				} else
					*ptr = aChar;
				ptr++;
				totalread ++;
			}	
			bytestoread -= numRead;
		}
//	}	
}


status_t
BTSWacomTablet::GetModel(char *buff)
{
	buff[0] = '\0';
	SendCommand("~#\r");
	snooze(100000);
	GetLine(buff,32);

	return B_NO_ERROR;
}

status_t
BTSWacomTablet::SetAlwaysTransmit(const int8)
{
	SendCommand("AS1\r");
	return B_NO_ERROR;
}


status_t
BTSWacomTablet::Reset()
{
	SendCommand("$");
	fPort.ClearInput();
	fPort.ClearOutput();
	return B_NO_ERROR;
}
