#include "wacom.h"
#include <InterfaceDefs.h>
#include <stdlib.h>
#include <stdio.h>
#include <GameKit.h>
#include "tablet.h"
#define MAXERRS 100
#include <iostream.h>
#include <fstream.h>
#include <string.h>


bool		WacomInputDevice::sActive = false;
int32		WacomInputDevice::sSpeed = 0;
thread_id	WacomInputDevice::sThread = B_ERROR;
bigtime_t 	gDblClickTime;

char version[]="$VER: wacom 1.0.3 (23 jun 2000)";
char copyright[]="Wacom penpartner driver © 1998,2000 by Olaf van Es <gfx@xs4all.nl> \n";

char buffer[24];
char comport[10];
float division = 4;
int slaap = 100;
char rotate [10] = "no";
bool doRotate;
int ytabmax = 3600;
bool loadtablet;

BInputServerDevice*
instantiate_input_device()
{
	// this is where it all starts
	// make sure this function is exported!

	return (new WacomInputDevice());
}


WacomInputDevice::WacomInputDevice()
	: BInputServerDevice()
{
	// register your device(s) with the input_server
	// the last device in the list should be NULL

	input_device_ref	wacomDevice			= { "Wacom Device",
													B_POINTING_DEVICE,
													NULL };
	input_device_ref	*wacomDeviceList[2]	= {&wacomDevice, NULL};

	RegisterDevices(wacomDeviceList);	
}


WacomInputDevice::~WacomInputDevice()
{
	// cleanup
}


status_t
WacomInputDevice::InitCheck()
{
	// do any init code that could fail here
	// you will be unloaded if you return false

  	ifstream(infile);
	infile.open("/boot/home/config/settings/gfx.wacom_settings", ios::in );
	if (!infile)	{
	  cerr << "cannot open gfx.wacom_settings\n";
	  // create a settings file for the next time
	  ofstream outfile("/boot/home/config/settings/gfx.wacom_settings", ios::out );
	  if (outfile) {
	  	outfile << "SerialPort serial1\n";
	  	outfile << "Division 4.2\n";
	  	outfile << "Delay 100\n";
	  	outfile << "Rotate no 3600\n";
	  	outfile.close();
	  	}
	loadtablet = false;  	
	goto einde;
	} 
  	
  	infile >> buffer; 
	if (strcmp ( buffer, "SerialPort") == 0 ){
		infile >> comport;
		if (strcmp (comport, "none") == 0) { 
			loadtablet = false;
			goto einde;
			}
		}
	infile >> buffer;
	if (strcmp ( buffer, "Division") == 0 ) {
		infile >> division;
		loadtablet = true;
		}
	
	infile >> buffer;
	if (strcmp ( buffer, "Delay") == 0 ) {
		infile >> slaap; // snooze
		}
	
	infile >> buffer;
	if (strcmp ( buffer, "Rotate") == 0) {
		infile >> rotate;
		if ( strcmp ( rotate, "yes" ) == 0) {
			doRotate = true;
			}
		infile >> ytabmax ; // max y value of tablet
		}
	
einde:
	
	if ( loadtablet == false)
		{
		return (false);
		}
	else
		{	
		return (BInputServerDevice::InitCheck());
		}
}


status_t
WacomInputDevice::SystemShuttingDown()
{
	// do any cleanup (ie. saving a settings file) when the
	// system is about to shut down

	return (BInputServerDevice::SystemShuttingDown());
}


status_t
WacomInputDevice::Start(
	const char	*device,
	void		*cookie)
{
	// start generating events
	// this is a hook function, it is called for you
	// (you should not call it yourself)

	sActive = true;
	get_mouse_speed(&sSpeed);
	sThread = spawn_thread(wack_em, device, B_DISPLAY_PRIORITY, this);

	resume_thread(sThread);

	return (B_NO_ERROR);
}


status_t
WacomInputDevice::Stop(
	const char	*device,
	void		*cookie)
{
	// stop generating events
	// this is a hook function, it is called for you
	// (you should not call it yourself)

	status_t err = B_OK;

	sActive = false;
	wait_for_thread(sThread, &err);
	sThread = B_ERROR;

	return (B_NO_ERROR);
}


status_t
WacomInputDevice::Control(
	const char	*device,
	void		*cookie,
	uint32		code,
	BMessage	*message)
{
	// respond to changes in the system
	// this is a hook function, it is called for you
	// (you should not call it yourself)

	switch (code) {
		case B_CLICK_SPEED_CHANGED:
			if (B_OK != get_click_speed(&gDblClickTime)) gDblClickTime = 250000;
			break;
			}

	return (B_NO_ERROR);
}


int32
WacomInputDevice::wack_em(
	void	*arg)
{
	int32   x, y;
	int32	oldx = 0, oldy = 0; 
	int8  bsp = 0, oldbsp = 0;
	int8 device; 
	int8 muis1, muis2, wisser;
	int32 mbuttons;
	int8 nabij;
	int8 vorig1 = 0;
	int8 vorig2 = 0; 
	float presbit;
	float xval,yval,maxx,maxy;
	
	uint32 aantalbytes;
	uint8 buf[]="0123456789";

	int32		clickcount = 0;
	bigtime_t	lastclick = 0;
	float 	lastClickedX = -1.0f, lastClickedY = -1.0f;
	float	deltaX = 0.0f, deltaY = 0.0f;

	WacomInputDevice *wacomDevice = (WacomInputDevice *)arg;

	BTSWacomTablet tablet(comport);
	tablet.Reset();
	tablet.SendCommand("#"); // wacom IV mode	
	snooze(100000);
	tablet.SendCommand("SP");
	tablet.SetAlwaysTransmit();
	
	if (B_OK != get_click_speed(&gDblClickTime)) gDblClickTime = 250000;
			

	while (wacomDevice->sActive) {
	if (loadtablet)
	{


	tablet.SendCommand("@");
	aantalbytes = tablet.fPort.Read(buf,7);

	if (buf[0] & 0x80)
	{
	device = buf[0] & 0x20;                    /* cursor or stylus ? */
	muis1 = buf[3] & 0x08;	// button 1
	muis2 = buf[3] & 0x10;	// button 2
	wisser = buf[3] & 0x20;	// eraser
	nabij = buf[0] & 0x40;	// proximity

	mbuttons = (muis1 ? 0x1 : 0x0)|(muis2 ? 0x2 : 0x0);

	x = (buf[0] & 0x3 ) << 14;    /* process array point[] to get one coordinate  */
	x |= buf[1] << 7;
	x |= buf[2];

	y = (buf[3] & 0x3) << 14;
	y |= buf[4] << 7;
	y |= buf[5];

	bsp = (buf[3] & 0x04 ) >> 2; // pressure
	bsp |= (buf[6] & 0x7F) << 1;

	presbit = (float)(bsp + 120) / 240.0;
		if (presbit > 1) presbit = 1.0f;
		if (presbit < 0) presbit = 0.0f;

	maxx=5040.0f*4.0f;
	maxy=3780.0f*4.0f;

	xval = ((float)(x)/(maxx / division))-0.005;
	yval = ((float)(y)/(maxy / division))-0.005; 


	if (nabij) 
	  	{
	  	
	  	if ((x != oldx) || (y != oldy))
	  	{
//	  		if (doRotate) {
//	  			tmpy = y;
//	  			y = x;
//	  			x = ytabmax - tmpy;
//	  			}
	  		
			bigtime_t now = system_time();

			if (now - lastclick < gDblClickTime) {
				deltaX = xval - lastClickedX; if (deltaX < 0) deltaX = -deltaX;
				deltaY = yval - lastClickedY; if (deltaY < 0) deltaY = -deltaY;
				if ((deltaX < (200.0 / maxx)) && (deltaY < (200.0 / maxy))) {
					xval = lastClickedX; yval = lastClickedY;
				}
			}
// click test


			BMessage	*event = new BMessage(B_MOUSE_MOVED);
			event->AddInt64("when", now);
			event->AddInt32("buttons", mbuttons);
			event->AddFloat("x", xval);
			event->AddFloat("y", yval);
			event->AddFloat("be:tablet_pressure", presbit);
			event->AddFloat("be:tablet_x", xval);
			event->AddFloat("be:tablet_y", yval);

			event->AddInt32("be:tablet_eraser", wisser);

//			if (intuos){
//				event->AddFloat("be:tablet_tilt_x", ftiltx);
//				event->AddFloat("be:tablet_tilt_y", ftilty);
//			}	
			wacomDevice->EnqueueMessage(event);
	  		
	  		
	  		
	  		
		}
		
		if ((muis1 && !vorig1) || (muis2 && !vorig2))
		{
			bigtime_t	now = system_time();
			BMessage	*event = new BMessage(B_MOUSE_DOWN);

			if ((now - lastclick > gDblClickTime) || ((deltaX > (200.0 / maxx)) && (deltaY > (200.0 / maxy)))){
				clickcount = 1; lastClickedX = xval; lastClickedY = yval;
			} else {
				++clickcount; xval = lastClickedX; yval = lastClickedY;
			}
			lastclick = now;

			event->AddInt64("when", now);
			event->AddInt32("buttons", mbuttons);
			event->AddFloat("x", xval);
			event->AddFloat("y", yval);
			event->AddFloat("be:tablet_pressure", presbit);
			event->AddInt32("clicks", clickcount);
			event->AddInt32("modifiers", modifiers());
			wacomDevice->EnqueueMessage(event);

		}

		if ((!muis1 && vorig1) || (!muis2 && vorig2))
		{
			bigtime_t	now = system_time();
			BMessage	*event = new BMessage(B_MOUSE_UP);
			event->AddInt64("when", now);
			event->AddInt32("buttons", mbuttons);
			event->AddFloat("x", xval);
			event->AddFloat("y", yval);
			event->AddInt32("clicks", 0x0);
			event->AddInt32("modifiers", modifiers());	
			wacomDevice->EnqueueMessage(event);
	  	}
			  	
		vorig1 = muis1;
		vorig2 = muis2;
		oldx = x;
		oldy = y;
		oldbsp = bsp;

		snooze(slaap);  // penpartner works with 100
						// artpad II needs higher value
		}
	else
		{
		snooze(50000); // when out of prox. sleep harder...
		}
	} // if 0x80
	else snooze (10000); 
		
	}// loadtablet
	else snooze (1000000);
	}// while
	
	
	return (B_OK);
}

